//****************************************************************************
//
//! \brief Opening a UDP server side socket and receiving data
//!
//! This function opens a UDP socket in Listen mode and waits for an incoming
//! UDP connection.
//!    If a socket connection is established then the function will try to
//!    read 1000 UDP packets from the connected client.
//!
//! \param[in]          port number on which the server will be listening on
//!
//! \return             0 on success, Negative value on Error.
//
//****************************************************************************

#define MV_CC3200_ERR   Report
#define MV_CC3200_DBG   Report

int BsdUdpServer(unsigned short usPort)
{
    SlSockAddrIn_t  sAddr;
    SlSockAddrIn_t  sLocalAddr;
    int             iCounter;
    int             iAddrSize;
    int             iSockID;
    int             iStatus;
    long            lLoopCount = 0;
    short           sTestBufLen;
    
    int ret = -1;
    char buf[1000] = {0};
    struct sockaddr_in remote_addr;
    socklen_t remote_size = sizeof(struct sockaddr);
    int len;
    fd_set rfds;
    struct timeval timeout;

    // filling the buffer
    for (iCounter=0 ; iCounter<BUF_SIZE ; iCounter++)
    {
        g_cBsdBuf[iCounter] = (char)(iCounter % 10);
    }

    sTestBufLen  = BUF_SIZE;
    //filling the UDP server socket address
    sLocalAddr.sin_family = SL_AF_INET;
    sLocalAddr.sin_port = sl_Htons((unsigned short)usPort);
    sLocalAddr.sin_addr.s_addr = 0;

    iAddrSize = sizeof(SlSockAddrIn_t);

    // creating a UDP socket
    iSockID = sl_Socket(SL_AF_INET,SL_SOCK_DGRAM, 0);
    if( iSockID < 0 )
    {
        // error
        ASSERT_ON_ERROR(SOCKET_CREATE_ERROR);
    }

    // binding the UDP socket to the UDP server address
    iStatus = sl_Bind(iSockID, (SlSockAddr_t *)&sLocalAddr, iAddrSize);
    if( iStatus < 0 )
    {
        // error
        sl_Close(iSockID);
        ASSERT_ON_ERROR(BIND_ERROR);
    }

    // no listen or accept is required as UDP is connectionless protocol
    /// waits for 1000 packets from a UDP client
    while (1)
    {
      FD_ZERO(&rfds);
      FD_SET(iSockID, &rfds);
      ret = select(iSockID+1, &rfds, NULL, NULL, NULL);
      if(ret < 0)
      {
        MV_CC3200_ERR("select error\r\n");
        break;
      }
      else if(ret == 0)
      {
        MV_CC3200_DBG("timeout\r\n");
        continue;
      }
      else
      {
        if(FD_ISSET(iSockID, &rfds))
        {
          if( (len=recvfrom(iSockID, buf, sizeof(buf), 0, (struct sockaddr *)&remote_addr, &remote_size)) < 0)
          {
            MV_CC3200_ERR("udp server recvfrom error\r\n");
            break;
          }
          MV_CC3200_DBG("received length is: %d\r\n\r\n", len);
          //MV_CC3200_DBG("receive data is:\r\n");
          //for(int i=0; i<len; i++)
            //MV_CC3200_DBG("0x%02x ", buf[i]);
          //MV_CC3200_DBG("\r\n");
        }
      }
    }
    //closing the socket after receiving 1000 packets
    sl_Close(iSockID);

    return SUCCESS;
}